#!/usr/bin/perl

use strict;
use warnings;

use Encode::JP;	# to import this module into an exe file which will be generated by PAR Packager

use Encode qw(encode decode);
use Encode::Guess;
use File::Spec::Functions qw(catfile);
use File::Path qw(mkpath);
use Text::CSV_XS;
use LWP::Simple;
use Getopt::Long qw(:config auto_help);
use Pod::Usage qw(pod2usage);

# Gets options and analyzes them.
my %options = (
	outdir => 'img',
	wait   => 5
);
Getopt::Long::GetOptions(\%options, qw(outdir=s ascii wait=i proxy=s useragent=s)) or pod2usage(2);
pod2usage(1) if ($options{help});

# Configures network settings. (Not checked whether it works...)
if ($options{proxy}) {
	$LWP::Simple::ua->proxy('http', $options{proxy});
}
if ($options{useragent}) {
	$LWP::Simple::ua->agent($options{useragent});
}

my $wait_sec = $options{wait};
$wait_sec = 5 if $wait_sec < 5;

# Opens and reads a CSV file to guess encoding.
open my $fh_tmp, '<', $ARGV[0] or die;
my $content = do {local $/; <$fh_tmp>};
close $fh_tmp;
my $enc = guess_encoding($content, qw/cp932 euc-jp/);
my $enc_name = $enc->name;

# Re-opens the CSV file in guessed encoding and analyzes it.
my @items;
my $csv = Text::CSV_XS->new({binary => 1});
open my $fh, "<:encoding($enc_name)", $ARGV[0] or die;
while (my $row = $csv->getline($fh)) {
	my $item_name = shift @$row;
	for my $str (@$row) {
		if ($str =~ /http:\/\/.*([0-9a-f]{40})/) {
			my $url = "http://hiroba.dqx.jp/dq_resource/itemicon/$1.png";
			push @items, {name => $item_name, url => $url};
			last;
		}
	}
}
close $fh;

# Gets and stores icon images.
my $finename_encoding = (index($^O, 'MSWin') == 0) ? 'cp932' : 'utf-8';
for my $item (@items) {
	# Makes a directory for images.
	unless (-d $options{outdir}) {
		mkpath($options{outdir});
	}

	# Gets an image from the DQX server.
	my $image_outpath = encode($finename_encoding, catfile($options{outdir}, "$item->{name}.png"));
	printf "GET %s -> %s: ", $item->{url}, $image_outpath;
	my $rescode = getstore($item->{url}, $image_outpath);
	printf "%d %s\n", $rescode, is_success($rescode) ? 'OK' : 'NG';

	sleep $wait_sec;
}

=head1 NAME

dqxitemicons - Gets the item icon images from the DRAGON QUEST web server.

=head1 SYNOPSIS

dqxitemicons [options] <CSV file>

  --outdir    A string of output directory for image files.
              Default is "img". (under the current directory)
  --wait      A value of waiting time in seconds for each download.
  --proxy     A string of proxy host name.
  --useragent A string of user agent.
  --help      Print this message.

  This script assumes that an input CSV file is the following format:

    <item name>, ... (zero or more values) ..., <the URL of the item>
    ...

  You can generate CSV file for this tool by using the following tool:
    http://dq10tools.appspot.com/item_list.html

=cut
